!
! bandlanczosdiag --- diagonalize band structure matrix using Thomas Sommerfelds
!       band Lanczos algorithm
! Christian Buth
! 08.08.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 14.02.2004 (Diagonalize band structure matrix fully)
!


!
! Include files
!

#include "assert.h"


!
! Subroutine
!

      subroutine bandlanczosdiag(am, ad, niter)
      use physconst
      use lattice
      use coadcdiag
      use coadcfile
      implicit none

      integer, intent(in) :: niter
      type(tbandsatkpoint), intent(inout) :: ad
      type(tcoadcmatrix), intent(inout) :: am

      integer i

      integer neigenv, ncartsv, iparam(10), icartsv(1)
      complex(dblprec), allocatable :: lanczosvectors(:,:,:), &
     &          Tmat(:,:), z(:,:), zev(:,:)
      complex(dblprec) ev(1,1)

      integer evlist(1)
      real(dblprec) thresh(3)
      integer, allocatable :: ipvt(:)
      complex(dblprec), allocatable :: a(:,:), e(:), e2(:)
      complex(dblprec), allocatable :: au(:,:), al(:,:), b(:)

      external hblanc, hbnddia


      write (*,*)
      write (*,*) 'Diagonalizing the band structure matrix...'



      neigenv = 0
      ncartsv = 0
      iparam(1) = 1
      iparam(2) = niter
      iparam(3) = 0
      iparam(4) = 0
      iparam(5) = 0
      iparam(6) = 0

      ! Allocate memory
      ad%numBands = niter
      allocate(ad%energies(niter), ad%polestrengths(niter))
      ad%energies(niter) = 0
      ad%polestrengths(niter) = 0
      allocate(lanczosvectors(am%ndim, am%main, 3), Tmat(am%main+1, niter))

      ! Generate canonical start vectors
      lanczosvectors = 0
      do i=1, am%main
        lanczosvectors(i, i, 2) = (1.0, 0.0)
      end do

      call hblanc(am%ndim, am%main, niter, neigenv, ncartsv, iparam, Tmat, &
     &     lanczosvectors, z, ev, icartsv, am)


       allocate(z(am%main+1, niter), zev(2*am%main, niter))
     !!!&         z(niter, am%main+1), zev(2*am%main, niter))

      thresh(1) = -20
      thresh(2) = 1
      thresh(3) = 0
      neigenv = 0
      evlist(1) = 0
      iparam(1) = 0

        allocate(au(niter,2*am%main+1), al(niter,2*am%main+1), b(niter))
        allocate(a(niter,am%main+1), e(niter), e2(niter), ipvt(niter))

      call hbnddia(niter, am%main, Tmat, z, ad%energies, e, e2, zev, thresh, &
     &                    neigenv, evlist, au, al, b, ipvt, iparam, ad%polestrengths, ad%numBands)

      deallocate(au, al, b)
      deallocate(a, e, e2, ipvt)
      deallocate(z, zev)
      deallocate(lanczosvectors, Tmat)
      end subroutine bandlanczosdiag
