!
! bandmatvec --- matrix times vector for the Dyson CO-ADC matrix
! Christian Buth
! 08.08.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 08.08.2004 (ipeamatvec taken as basis)
! Version 1.10: 27.08.2004 (Bug in off-diagonal element treatment removed)
! Version 1.20: 05.10.2004 (Bug in complex conjugation removed)
!


!
! Subroutine
!

      subroutine bandmatvec(am, bw, vpre, vpost)
      use coadcfile
      use coadcdiag
      implicit none

      integer, intent(in) :: bw
      complex(dblprec), intent(in) :: vpre(:,:)
      complex(dblprec), intent(out) :: vpost(:,:)
      type(tcoadcmatrix), intent(inout) :: am
      logical ameof
      integer i, j

      call amrewind(am)
      vpost = 0

      ! Multiply vector with diagonal elements of CO-ADC matrix
      do
        call readmatrixbuffer(am, .true., ameof)
        if (ameof) then
          exit
        end if

        do i=1, am%nwd
          do j=1, bw
            ! CO-ADC matrix is complex Hermitian
! wrong           vpost(am%idx(i), j) = vpost(am%idx(i), j)            &
!     &                    + conjg(am%lattice%factor(am%latd(i))) &
!     &                    * am%diag(i) * vpre(am%idx(i), j)
            vpost(am%idx(i), j) = vpost(am%idx(i), j)            &
     &                    + am%lattice%factor(am%latd(i)) &
     &                    * am%diag(i) * vpre(am%idx(i), j)
          end do
        end do
      end do

      ! Multiply vector with off-diagonal elements of CO-ADC matrix
!      write (*,*) 'The RIGHT matrix from the paper!!!!!!!!!!!'
      do
        call readmatrixbuffer(am, .false., ameof)
        if (ameof) then
          exit
        end if

        ! CO-ADC matrix is complex Hermitian where the upper triangle is stored
        do i=1, am%nwo
          !if (am%iox(i) >= am%jox(i)) then
          !  write (*,*) 'Wrong matrix format! Upper triangle should &
          !              &be stored, but am%iox(i) >= am%jox(i)!'
          !  stop 'Upper triangle!'
          !end if

          if (am%iox(i) <= bw .and. am%jox(i) > bw) then

            do j=1, bw
              vpost(am%iox(i), j) = vpost(am%iox(i), j)            &
     &                     + conjg(am%lattice%factor(am%lato(i)))  &
     &                     * cmplx(am%off(i), 0, dblprec) * vpre(am%jox(i), j)
              vpost(am%jox(i), j) = vpost(am%jox(i), j)            &
     &                     +       am%lattice%factor(am%lato(i))   &
     &                     * cmplx(am%off(i), 0, dblprec) * vpre(am%iox(i), j)
            end do

          else

            do j=1, bw
              vpost(am%iox(i), j) = vpost(am%iox(i), j)            &
     &                     +       am%lattice%factor(am%lato(i))   &
     &                     * cmplx(am%off(i), 0, dblprec) * vpre(am%jox(i), j)
              vpost(am%jox(i), j) = vpost(am%jox(i), j)            &
     &                     + conjg(am%lattice%factor(am%lato(i)))  &
     &                     * cmplx(am%off(i), 0, dblprec) * vpre(am%iox(i), j)
            end do

          end if
        end do
      end do
      end subroutine bandmatvec