!
! ipeafulldiag --- diagonalize the ADC matrix fully
! Christian Buth
! 13.02.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 13.02.2004 (Diagonalize the ADC matrix fully)
! Version 1.10: 29.02.2004 (Integrated into adcbands module)
! Version 1.20: 02.03.2004 (Optional exact treatment of the non-diagonal Fock matrix)
! Version 1.30: 03.03.2004 (Optional treatment of the non-diagonal Fock matrix at ADC(3) level)
! Version 1.40: 20.06.2004 (Matrix generation code replaced by matrix reader)
!


      subroutine ipeafulldiag(am, ad)
      use physconst
      use lattice
      use coadcdiag
      use coadcfile
      implicit none

      type(tbandsatkpoint), intent(out) :: ad
      type(tcoadcmatrix), intent(inout) :: am

      logical ameof

      integer i, j, &
     &        info, lwork, nonzero, numADCBands, numUpperTriangle

      real(dblprec) rx, sparsity
      real(dblprec), allocatable :: work(:),        &
     &                              bandstruct(:,:)

      numADCBands = am%ndim
      lwork = 3 * numADCBands - 1

      ! Allocate memory
      ad%numBands = am%ndim
      allocate(bandstruct(numADCBands,numADCBands), &
     &         work(lwork),                         &
     &         ad%energies(am%ndim),                &
     &         ad%polestrengths(am%ndim))

      ! Get diagonal elements of CO-ADC matrix
      bandstruct = 0
      do
        call readmatrixbuffer(am, .true., ameof)
        if (ameof) then
          exit
        end if

        do i=1, am%nwd
          bandstruct(am%idx(i), am%idx(i)) = am%diag(i)
        end do
      end do

      ! Get off-diagonal elements of CO-ADC matrix
      do
        call readmatrixbuffer(am, .false., ameof)
        if (ameof) then
          exit
        end if

        do i=1, am%nwo
          bandstruct(am%iox(i), am%jox(i)) = am%off(i)
          bandstruct(am%jox(i), am%iox(i)) = am%off(i)
        end do
      end do


      ! Sparsity of the band structure matrix
      nonzero = 0
      do i=1, numADCBands
        do j=i, numADCBands
          if (bandstruct(i,j) /= 0) then
            nonzero = nonzero + 1
          end if
        end do
      end do
      ! The zero coupling matrix between 2p1h block and 2h1p block is subtracted
      !!!numUpperTriangle = numADCBands * (numADCBands + 1) / 2 - num2p1h * num2h1p
      numUpperTriangle = numADCBands * (numADCBands + 1) / 2
      sparsity = real(numUpperTriangle - nonzero) / real(numUpperTriangle) * 100.0

      write (*,'(A,F6.2,A)') 'Sparsity of ADC matrix ', sparsity, '%'


      ! Diagonalize ADC matrix
      call dsyev('V', 'U', numADCBands, bandstruct, numADCBands, ad%energies, work, lwork, info)

      if (info.ne.0) then
          write (*,*)
          write (*,*) 'dsyev returned error while diagonalizing the Fock matrix: info = ', info
          stop 'CO-ADC execution error'
      end if

      ! Write ionization potentials
!      write (*,*)
!      write (*,*) 'The band structure'
!      write (*,*) 'IP                           Pole strength'
!      write (*,*)
!      write (*,*)
      do i=1, numADCBands
        rx = 0
        ! Francesco Tarantelli's definition of ADC pole strength
        do j=1, am%main
          rx = rx + bandstruct(j,i)**2
        end do
        ad%polestrengths(i) = rx

        if (rx > 0.2) then
          write (*,*) -ad%energies(i), -ad%energies(i) * HartreetoeV, rx
        end if
      end do

      deallocate(bandstruct, work)
      end subroutine ipeafulldiag
