!
! ipealanczosdiag --- diagonalize the ADC matrix with a band Lanczos algorithm
! Christian Buth
! 27.03.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 24.08.2004 (bandlanczosdiag taken as basis)
!


!
! Subroutine
!
      subroutine ipealanczosdiag(am, ad)
      use physconst
      use lattice
      use coadcdiag
      use coadcfile
      implicit none

      type(tbandsatkpoint), intent(inout) :: ad
      type(tcoadcmatrix), intent(inout) :: am

      integer i, lwork, lrwork, numADCBands

      integer niter, neigenv, ncartsv, iparam(10), icartsv(1)
      real(dblprec), allocatable :: lanczosvectors(:,:,:), &
     &          Tmat(:,:), z(:,:), zev(:,:)
      real(dblprec) ev(1,1)

      integer evlist(1)
      real(dblprec) thresh(3)
      real(dblprec), allocatable :: a(:,:), e(:), e2(:)
      real(dblprec), allocatable :: au(:,:), al(:,:), b(:)
      integer, allocatable :: ipvt(:)

      external rblanc, rbnddia

      numADCBands = am%ndim
      lwork = 2 * numADCBands - 1
      lrwork = 3 * numADCBands - 2


      write (*,*)
      write (*,*) 'Diagonalizing the band structure matrix...'



      niter = 80
      niter = 950
      neigenv = 0
      ncartsv = 0
      iparam(1) = 1
      iparam(2) = niter
      iparam(3) = 0
      iparam(4) = 0
      iparam(5) = 0
      iparam(6) = 0

      ! Allocate memory
      ad%numBands = am%ndim
      allocate(lanczosvectors(am%ndim, am%main, 3), Tmat(am%main+1, niter))

      ! Generate canonical start vectors
      lanczosvectors = 0
      do i=1, am%main
        lanczosvectors(i, i, 2) = 1
      end do

      call rblanc(am%ndim, am%main, niter, neigenv, ncartsv, iparam, Tmat, &
     &     lanczosvectors, z, ev, icartsv, am)

      ad%numBands = niter
      allocate(ad%energies(ad%numBands), ad%polestrengths(ad%numBands), &
     &         z(am%main+1, niter), zev(2*am%main, niter))
     !!!&         z(niter, am%main+1), zev(2*am%main, niter))

      thresh(1) = -20
      thresh(2) = 1
      thresh(3) = 0
      neigenv = 0
      evlist(1) = 0
      iparam(1) = 0

        allocate(au(niter,2*am%main+1), al(niter,2*am%main+1), b(niter))
        allocate(a(niter,am%main+1), e(niter), e2(niter), ipvt(niter))

      call rbnddia(niter, am%main, Tmat, a, ad%energies, e, e2, zev, thresh, &
     &                    neigenv, evlist, au, al, b, ipvt, iparam, ad%polestrengths)


      ! Write ionization potentials
      write (*,*)
      write (*,*) 'The band structure at k-point ', am%lattice%crystalmomentum
      write (*,*) 'IP [a.u.]             IP [eV]             Pole strength'

/**********
      do i=1, numADCBands
        rx = 0
        do j=1, am%main
          rx = rx + bandstruct(j,i) * conjg(bandstruct(j,i))
        end do
        ad%polestrengths(i) = rx

        if (rx > 0.3) then
          write (*,*) ad%eigenvalues(i), ad%eigenvalues(i) * HartreetoeV, rx
        end if
      end do
**********/

      write (*,*)
!      deallocate(bandstruct, work, rwork)
      end subroutine ipealanczosdiag
