!
! cabhd --- read real-symmetric CO-ADC matrix file and write out Heidelberg's
!       ADC matrix file
! Christian Buth
! 04.08.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 04.04.2004 (Diagonalize ADC matrices with ARPACK)
!


!
! Program
!

      program cabhd
      use coadcfile
      implicit none

      character(1024) fileADCip,      &                 ! Negative of ADC eigenvalues
     &                fileCOADCBands                    ! CO-ADC band structure
      integer, parameter :: fadcdia = 50, &
     &                      fadcoff = 51
      type(tcoadcmatrix) am

/*********
      namelist /coadc/ !numBrillouinPaths
     &               !numUnitCells, numWannierUnitCells, startOccupied, endVirtual, &
     &               !firstBand, lastBand, polethresh, periodicityDim, &
     &               !fileFockip, fileADCip, fileFockBands, fileCOADCBands, &
     &               !fileBands, fileFockAll, &
     &               !fileFockTrans, fileVBucket, fileVijkl, fileVIndex, &
*****/

      ! Greeting
      write (*,*) 'Crystal orbital algebraic diagrammatic constuction eigenvalue solver'
      write (*,*) '(c) 2004 Christian Buth'
      write (*,*)
      write (*,*)


!      read (*,coadc)


      ! Open files for the diagonal and the off-diagonal matrix
      ! elements of the CO-ADC matrix
      write (*,*) 'Open files for the diagonal and the off-diagonal matrix elements', &
     &            'of the CO-ADC matrix'
      am%readfile = .true.
      am%fadccom  = 393
      am%fadcdia  = 394
      am%fadcoff  = 395
      call opencoadcfile(am)

      ! Carry out CO-ADC calculation
      select case (am%periodicity)
        case (0)
          write (*,*) 'The system is a molecule'
        case (1)
          write (*,*) 'The system is a polymer'
        case (2)
          write (*,*) 'The system is a surface'
        case (3)
          write (*,*) 'The system is a crystal'
      end select

      if (am%periodicity == 0) then

        call convertmatrix

      else
        write (*,*)
        write (*,*) 'The band structure matrix is for a periodic ',     &
     &              'system and, hence, unsupported by the Heidelberg', &
     &              ' eigenvalue solver!'
        stop 'Conversion not possible!'
      end if

      call closecoadcfile(am)


      write (*,*)
      write (*,*)
      write (*,*)
      write (*,*) 'Excecution sucessful'
      write (*,*)
      write (*,*) 'Da steh ich nun, ich armer Tor!'
      write (*,*) 'Und bin so klug als wie zuvor;'
      write (*,*) '                Goethe, Faust I'


      contains

!
! convertmatrix ---
        ! Read CO-ADC matrix elements, write Heidelberg ADC matrix format
!

      subroutine convertmatrix
      implicit none

      logical ameof
      integer matrixformat, i
      real(dblprec), allocatable :: adcdiagonal(:)

      matrixformat = 0

      ! Allocate memory
      allocate(adcdiagonal(am%ndim))

      ! Get diagonal elements of CO-ADC matrix
      do
        call readmatrixbuffer(am, .true., ameof)
        if (ameof) then
          exit
        end if

        do i=1, am%nwd
          adcdiagonal(am%idx(i)) = am%diag(i)
        end do
      end do

      ! Put header and matrix diagonal in Francesco Tarantelli's or
      ! Thomas Sommerfeld's matrix file format
      open (unit=fadcdia, file='hddia_1', status='replace',          &
     &      action='write', form='unformatted', access='sequential')

      ! Francesco Tarantelli's diagonalizer files
      ! Matrix files are: Filedia_hd1  and Fileoff_hd1
      !
      ! Assumed structure of Filedia_hd1 :
      ! real*8 diag
      ! integer*4 main,nrec,ndim
      ! main,nrec,ndim,diag(ndim)
      !
      ! Assumed structure of Fileoff_hd1 :
      ! real*8 a
      ! integer ix,jx,nw,nbuf
      ! a(nbuf),ix(nbuf),jx(nbuf),nw
      ! for nrec records. nbuf must be input
      ! nbuf set to:       10000
      !
      ! Same format as in Thomas Sommerfeld diagonalizer files of fformat=2
      select case(matrixformat)
        case  (0,2)
          write (*,*) 'am%main, am%nreco, am%ndim'
          write (*,*) am%main, am%nreco, am%ndim
          write (unit=fadcdia) am%main, am%nreco, am%ndim, &
     &                         (adcdiagonal(i), i=1, am%ndim)

      ! Thomas Sommerfeld diagonalizer files of fformat=1
        case(1)
          i = 1           ! Real part of matrix (there is no imaginary part)
          write (*,*) 'am%ndim, am%nbuf, am%nreco, i'
          write (*,*) am%ndim, am%nbuf, am%nreco, i
          write (unit=fadcdia) am%ndim, am%nbuf, am%nreco, i
          write (unit=fadcdia) adcdiagonal
      end select
      close (fadcdia)
      deallocate(adcdiagonal)

      ! Get off-diagonal elements of CO-ADC matrix
      open (unit=fadcoff, file='hdoff_1', status='replace',         &
     &      action='write', form='unformatted', access='sequential')
      do
        call readmatrixbuffer(am, .false., ameof)
        if (ameof) then
          exit
        end if
        write (unit=fadcoff) am%off, am%iox, am%jox, am%nwo
      end do
      close(unit=fadcoff)

      end subroutine convertmatrix
      end program cabhd
