!
! adcbands --- module that contains all information to be used in
!       conjunction with CO-ADC band structures
! Christian Buth
! 10.02.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 10.02.2004 (CO-ADC matrix generation)
! Version 1.10: 28.02.2004 (Active orbitals can be specified for correlation calculations)
! Version 1.20: 08.07.2004 (Diagonalization moved to coadcdiag, configuration selection)
!


!
! Include files
!

#include "assert.h"


!
! Module
!

      module adcbands
      use fortrantl
      use lattice
      implicit none


!
! Types
!

      type tconfiguration
        integer g1, g2, cj, ck, cl
      end type tconfiguration


      type tconfigurationset
        integer numConfigs
        type(tconfiguration), pointer :: configs(:)
      end type tconfigurationset

#include "tcoadc.h"


!
! Interface
!

      interface
      subroutine adcmatrix(wb, cm, fockeigenvectors, fockeigenvalues, &
     &           exactoffdiag, fockpt3)
        use wannier
        real(dblprec) fockeigenvectors(:,:), &
     &                fockeigenvalues(:)
        logical, intent(in) :: exactoffdiag, fockpt3
#include "tcoadc.h"
        type(twannier), intent(in) :: wb
        type(tcoadc), intent(in) :: cm
      end subroutine adcmatrix


      subroutine twointdecay
      end subroutine twointdecay
      end interface


!
! Procedures
!

      contains


!
! initcoadc --- initialize tcoadc type
!

      subroutine initcoadc(cm, numorbs, numocc, startocc, endvirt, numucs)
      implicit none

      integer, intent(in) :: numorbs, numocc, startocc, endvirt, numucs
      type(tcoadc), intent(out) :: cm

      cm%startOccupied = startocc
      cm%endVirtual    = endvirt
      cm%numUnitCells  = numucs
      cm%numOrbitals = endvirt - startocc + 1
      cm%numOccupied = numocc - startocc + 1
      cm%numVirtual  = endvirt  - numocc

      write (*,*) 'Wannier parameters are: numOccupied ', numocc
      write (*,*) 'CO-ADC parameters are: startOccupied, endVirtual, numOrbitals, numOccupied, numVirtual, numUnitCells'
      write (*,*) cm%startOccupied, cm%endVirtual, cm%numOrbitals, cm%numOccupied, cm%numVirtual, cm%numUnitCells

      cm%num2p1h  = cm%numUnitCells * cm%numVirtual  * cm%numOccupied &
     &              * (cm%numUnitCells * cm%numVirtual  + cm%numUnitCells - 1)
      cm%num2h1p  = cm%numUnitCells * cm%numOccupied * cm%numVirtual  &
     &              * (cm%numUnitCells * cm%numOccupied + cm%numUnitCells - 1)
      ! No excitations are generated involving the first cm%startOccupied orbitals or
      ! the last numorbs - endvirt + 1, but coupling matrices to them are
      ! nevertheless generated.
      cm%numBands = numorbs + cm%num2p1h + cm%num2h1p
      end subroutine initcoadc


!
! exitcoadc--- clean up tcoadc type
!

      subroutine exitcoadc(cm)
      implicit none

      type(tcoadc), intent(inout) :: cm
      end subroutine exitcoadc
      end module adcbands
