!
! fockall --- load the FOCKALL matrix into main memory
! Martin Albrecht
!
!
! Copyright (c) 2003 Martin Albrecht
! Copyright (c) 2004-2006 Christian Buth
!
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version.
!
! This library is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! Lesser General Public License for more details.
!
! You should have received a copy of the GNU Lesser General Public
! License along with this library; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
!
!
! Version 1.00: (extracted code from Martin Albrecht from incread2.f of Greens program)
! Version 1.10: 04.07.2004 (Correct treatment of ntpqpseudo of Wannier, core state purification)
!


      subroutine fockall(wb, fileFockAll)
      use wannier
      use Vintegrals
!      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      implicit none

      integer i, j, k, istat, ivect, iorb, jorb, iunit, &
     &        nvec, iv, nx, ny, nz, na, nb, nc, mopca,  &
     &        mopc, io

      character(len=*), intent(in) :: fileFockAll
      type(twannier), intent(inout) :: wb



! CB #include "param_greens"
! CB #include "common_greens"


! ---------- basis vectors -------------


      select case (wb%periodicity)
        case (0:2)
          ! No transformation of direct space lattice indices
          abas(1,1)=1 ; abas(2,1)=0 ; abas(3,1)=0
          abas(1,2)=0 ; abas(2,2)=1 ; abas(3,2)=0
          abas(1,3)=0 ; abas(2,3)=0 ; abas(3,3)=1
        case (3)
          ! FCC Transformation auf kartesische Koordinaten
          abas(1,1)=0 ; abas(2,1)=1 ; abas(3,1)=1
          abas(1,2)=1 ; abas(2,2)=0 ; abas(3,2)=1
          abas(1,3)=1 ; abas(2,3)=1 ; abas(3,3)=0
        case default
          write (*,*) 'The periodicity of the system must be 0, 1, 2 or 3!'
          stop 'fockall: system periodicity wrong!'
      end select



      do i=1,mvector
        ivec(i)=0
        ivgroup(i)=0
        do k=1,3
          nvlist(k,i)=0
        end do
      end do

      do i=1,mvector
        do j=1,mvector
          ivecsym(i,j)=1
        end do
      end do

      do istat=1,2
        do ivect=1,mvector
          do iorb=1,morbit
            do jorb=1,morbit
              Fockmat(istat,ivect,iorb,jorb)=0.D0
            end do
          end do
        end do
      end do




      iunit  = 900
      icheck = 1                ! Debuginformationen ausgeben
      isockmat =-1              ! berlappmatrix einlesen aber nicht benutzen


!
! Essezieller Teil, um die FOCKALL Matrix einzulesen!!!
!
! Vermutung: ntpqpseudo ist der Radius des C + E1 Clusters in Wannier,
!       siehe fockgenerator(_virtuals.f) in Wannier. Meine nderung bewirkt,
!       dass mzell Fockmatrizen geschrieben werden
! mzell : Zahl der Zellen in C + E1
! moccx, morbx : pro Einheitszelle
!
      open (unit=iunit,file=fileFockAll,form='unformatted',action='read')
      read (iunit) ntpqpseudo,moccx,morbx,mzell
      write (iout,*)'ntpqpseudo,moccx,morbx,mzell= ', &
     &              ntpqpseudo,moccx,morbx,mzell
!      mzell = ntpqpseudo               !CB
      nvec = ntpqpseudo                !CB
!      nvec = mzell                      !CB
!      nvec = max(ntpqpseudo, mzell)    !CB
      ivecmax = mvector                 !CB
      nveccut=min(nvec,ivecmax)
      if (nvec .gt. ivecmax) then          !CB
        stop 'nvec .gt. ivecmax'
      end if

      if(nveccut.gt.mvector)then
        write(iout,*)'nveccut>mvector: ',nveccut,mvector
        write(iout,*)'=> setting nveccut=mvector= ',mvector
        nveccut=mvector                                 ! Anzahl Gittervektoren in dem Cluster von Wannier
                                                        ! sollte es sein, jedoch ist es der Grenparameter im
                                                        ! common block mvector
      endif

      do istat=1,2
        do ivect=1,mvector
          do iorb=1,morbit
            do jorb=1,morbit
              Fockmat(istat,ivect,iorb,jorb)=0.D0
              Sockmat(istat,ivect,iorb,jorb)=0.D0       ! berlappmatrix zwischen alle Orbitalen, geblockt
                                                        ! in Besetzt und Unbesetzt
            end do
          end do
        end do
      end do

      ! Initialize direct lattice information
      wb%numClusterUnitCells = mzell                       ! Wannier Rechnung unterliegender Cluster
      wb%numFockAllUnitCells = ntpqpseudo                  ! Einheitszellen fr die Fockallmatrix Berechnung
      ! CB wieder entfernt numFockAllUnitCells = mzell
      ! CB wieder entfernt numFockAllUnitCells = mzell
      allocate(wb%idirect(nvec), wb%cdirect(nvec))

      ! Alle Gittervektoren durchlaufen
      write (*,*)
      write (*,*) 'Lattice vectors of direct space Fock matrices'
      write (*,*) 'Cartesian coordiantes        Lattice vector indices'

      do iv=1, nveccut
        read(iunit) nx,ny,nz                             ! R vector indizes lesen
        na=nx*abas(1,1)+ny*abas(1,2)+nz*abas(1,3)       ! R vector kartesische indices
        nb=nx*abas(2,1)+ny*abas(2,2)+nz*abas(2,3)
        nc=nx*abas(3,1)+ny*abas(3,2)+nz*abas(3,3)
        write (*,'(I4,A,6(I4,A))') iv, ' (',nx, ',',ny, ',', nz, ')             (', na, ',', nb, ',', nc, ')'

        ! Store cartesian lattice vectors
!        wb%idirect(iv) = tlatticeindex(na,nb,nc)        ! Direct space lattice vectors in terms of
        wb%idirect(iv)%i(1) = na        ! Direct space lattice vectors in terms of
        wb%idirect(iv)%i(2) = nb
        wb%idirect(iv)%i(3) = nc
                ! the basis vectors of the primitive unit cell
!        wb%cdirect(iv) = tlatticevector(nx,ny,nz)        ! Direct space lattice vectors in terms of
                ! cartesian coordinates
        wb%cdirect(iv)%v = (/nx,ny,nz/)        ! Direct space lattice
                                    ! vectors in terms of cartesian coordinates

          nvlist(1,iv)=nx
          nvlist(2,iv)=ny
          nvlist(3,iv)=nz

        mopca=1
        mopc=moccx

        ! Besetzten Teil der Fockmatrix zu Gittervektor R einlesen
                                                                ! Fockmatrix hat 4 indizes
        read(iunit)((Fockmat(1,iv,iorb,jorb),   &
     &                  iorb=mopca,mopc),jorb=mopca,mopc)       ! 1. : =1 heit besetzt

        if((isockmat.eq.1).or.(isockmat.eq.-1)) then             !      =2 heit unbesetzt
                                                                 ! 2. : Gittervektor R
          read(iunit)((Sockmat(1,iv,iorb,jorb), &
     &                     iorb=mopca,mopc),jorb=mopca,mopc)    ! 3. , 4. : entangled band indizes
        end if

        mopca=moccx+1
        mopc=morbx

        ! Unbesetzten Teil der Fockmatrix zu Gittervektor R einlesen
                                                                        ! Virtuelle band indizes werden wieder
        read(iunit)((Fockmat(2,iv,iorb-moccx,jorb-moccx),   &
     &                  iorb=mopca,mopc),jorb=mopca,mopc)               ! bei 1 beginnend nummeriert

        if((isockmat.eq.1).or.(isockmat.eq.-1)) then
          read(iunit)((Sockmat(2,iv,iorb-moccx,jorb-moccx), &
     &                    iorb=mopca,mopc),jorb=mopca,mopc)
        end if

      end do    ! Fockmatrix zum nchsten Gittervektor einlesen
      close(unit=iunit)


      if(icheck.gt.0)then
        write(iout,*)'sr incread2: writeout of Fockmatrix'
        write(iout,*)'(diagonal only)'
        do io=1,moccx
          write(iout,*)io,fockmat(1,1,io,io)
        end do
        write(iout,*)' '
        do io=1, morbx - moccx
          write(iout,*)io,fockmat(2,1,io,io)
        end do
      endif

      ! Bei LiF scheie
      !call purifiycorestates


      contains

!
! purifiycorestates --- eliminate off-diagonal elements of core states
!

      subroutine purifiycorestates
      implicit none

      integer numOrbs, coreOrb, numCoreOrbs, orb, R

      numOrbs = morbit
      numCoreOrbs = 1

      ! Delete off-diagonal matrix elements in neighbouring unit cells for core orbitals
      do coreOrb=1, numCoreOrbs
      !  do orb=1, numOrbs
          do R=2, wb%numFockAllUnitCells
        orb=coreOrb
            Fockmat(1, R, coreOrb, orb) = 0
            Fockmat(1, R, orb, coreOrb) = 0
          end do
      !  end do
      end do
      end subroutine purifiycorestates
      end subroutine fockall
