!
! wannierfock --- diagonalize the FOCKALL matrix
! Christian Buth
! 10.02.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 10.02.2004 (Took readhf.f code and diagonalize the Fock matrix)
! Version 1.10: 14.02.2004 (Lattice sum implemented)
!


      subroutine wannierfock(wb, kpoint, eigenvalues)
      use mathtl
      use wannier
      implicit none

      real(dblprec), intent(out) :: eigenvalues(:)              ! Eigenvalues of the Fock matrix
      type(tlatticevector), intent(in) :: kpoint
      type(twannier), intent(in) :: wb

      integer i, j, R, info

      real(dblprec), allocatable :: rwork(:)
      complex(dblprec), allocatable :: Fock(:),         &       ! Upper triangle of the Fock matrix
     &                                 BandMixing(:,:), &       ! Band mixing matrix
     &                                 work(:)

      external zhpev


      ! Build the Fock matrix
      allocate(Fock(wb%numOrbitals * (wb%numOrbitals + 1) / 2), &
     &         BandMixing(wb%numOrbitals, wb%numOrbitals),      &
     &         work(2 * wb%numOrbitals - 1),                    &
     &         rwork(3 * wb%numOrbitals - 2))


      do j = 1, wb%numOrbitals
        do i = 1, j
          Fock(i + j * (j-1) / 2) = 0
          do R = 1, wb%numFockAllUnitCells
            Fock(i + j * (j-1) / 2) = Fock(i + j * (j-1) / 2) + latticefactor(kpoint, wb%cdirect(R)) &
     &         * cmplx(fallfind(i, R, j), 0, dblprec)
          end do
        end do
      end do

#ifdef trash
      if (periodicityDim == 1) then
        write (*,*)
        write (*,*) 'Difference of FOCKALL and Fock matrix'
        do j = 1, numWannierOrbitals
          do i = 1, j-1
            if (abs(Fock(i + j * (j-1) / 2) - ffind(i,j)) > 1.0D-6) then
              write (*,*) 'Fij', i, j, abs(Fock(i + j * (j-1) / 2) - ffind(i,j))
            end if
          end do
        end do
      end if

      write (*,*)
      write (*,*) 'Zero off-diagonal elements of Fock matrix'
      do j = 1, numWannierOrbitals
        do i = 1, j-1
          if (abs(ffind(i,j)) > 1.0D-6) then
            write (*,*) 'Non zero i,j ', i, j, abs(ffind(i,j))
          end if
        end do
      end do
#endif

      ! Diagonalize the Fock matrix
      call zhpev('V', 'U', wb%numOrbitals, Fock, eigenvalues, BandMixing, wb%numOrbitals, work, rwork, info)

      if (info.ne.0) then
          write (*,*)
          write (*,*) 'zhpev returned error while diagonalizing the Fock matrix: info = ', info
          stop 'CO-ADC execution error'
      end if

#ifdef trash
      if (periodicityDim == 1) then
        write (*,*)
        call fockmolval(work)
        write (*,*)
        write (*,*) 'Difference of eigenvalues between FOCKALL and Fock matrix'
        do i = 1, wb%numOrbitals
          if (abs(eigenvalues(i) - work(i)) > 1.0D-10) then
            write (*,*) 'Fii', i, abs(eigenvalues(i) - work(i))
          end if
        end do
      end if
#endif

      ! Determine the band mixing matrix for the chosen k-point from the eigenvectors of the
      ! symmetry adapted truncated Fock matrix (FOCKALL)
      BandMixing = conjg(transpose(BandMixing))

      ! Determine the full symmetry adapted Fock matrix from the Wannier Hartree-Fock band structure
/********************
      FockSymmetryAdapted = 0
      do i=1, wb%numOrbitals
        FockSymmetryAdapted(i,i) = wb%bands(i,kpoint)
      end do
      FockSymmetryAdapted = matmul(conjg(transpose(BandMixing)), &
     &                                          matmul(FockSymmetryAdapted,BandMixing))
***********************/
      deallocate(Fock,BandMixing,work,rwork)
      end subroutine wannierfock
