!
! zbandr --- this subroutine reduces a complex symmetric band matrix
!       to a complex symmetric tridiagonal matrix
! Thomas Sommerfeld based on a real symmetric code by Hans-Dieter Meyer
! Planet PKS version by Christian Buth
! 26 July 2004
!
!
! Copyright (c) 2003 Hans-Dieter Meyer
! Copyright (c) 2003 Thomas Sommerfeld
! Copyright (c) 2004-2006 Christian Buth
!
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version.
!
! This library is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! Lesser General Public License for more details.
!
! You should have received a copy of the GNU Lesser General Public
! License along with this library; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
!


!     -------------------------------------------------
!     complex version of the EISPACK bandr routine
!
!     the band matrix a is transformed to a tridiagonal form
!     n = dim(a)
!     m = halfbandwidth
!     nm = Anzahl der zu berechnenden Reihen von z
!     berechnet werden die ersten und letzten nm/2 Reihen
!     z = orthogonal transformation matrix
!     ---------------------------------------------------

      subroutine zbandr(nm,n,m,a,matz,z)
      use lanczos
      implicit none

      integer n, m, nm
      complex(dblprec) a(1:n,0:m), z(nm,n)
      logical matz

      integer j, k, l, r, maxr, maxl, ugl, nmd2
      integer nm2, kpr, jm1, jmmm1, jml, jmlm1, lp1, jm2
      complex(dblprec) b, c, s, c2, s2, cs, u, u1, g
      complex(dblprec) csa

      if (matz) then
         nmd2 = nm / 2
         do 10 j = 1, nmd2
            do 12 k = 1, n
               z(j,k) = (0.0d0, 0.0d0)
 12         continue
            z(j,j) = (1.0d0, 0.0d0)
 10      continue
         nmd2 = nmd2 + 1
         do 20 j = nmd2, nm
            do 22 k = 1, n
               z(j,k) = (0.0d0, 0.0d0)
 22         continue
            z(j,j - nm + n) = (1.0d0, 0.0d0)
 20      continue
      end if

      nm2 = n - 2
      do 100 k = 1, nm2
         if (n - k .le. m) then
            maxr = n - k
         else
            maxr = m
         end if

         do 200 r = maxr, 2, -1
            kpr = k + r

            do 300 j = kpr, n, m
               jm1 = j - 1
               jmmm1 = j - m - 1
               if (j .eq. kpr) then
                  if (a(k,r) .eq. (0.0d0, 0.0d0)) then
                     goto 200
                  endif
                  b = -a(k,r-1) / a(k,r)
                  ugl = k
               else
                  if (g .eq. (0.0d0, 0.0d0)) then
                     goto 200
                  endif
                  b = -a(jmmm1,m) / g
                  ugl = j - m
               end if

               s = (1.0d0, 0.0d0) / zsqrt((1.0d0, 0.0d0) + b**2)
               c = b * s
               c2 = c**2
               s2 = s**2
               cs = c*s
               csa = 2*cs*a(jm1,1)
               u  = c2*a(jm1,0) - csa + s2*a(j,0)
               u1 = s2*a(jm1,0) + csa + c2*a(j,0)
               a(jm1,1) = cs*(a(jm1,0)-a(j,0)) + (c2-s2)*a(jm1,1)
               a(jm1,0) = u
               a(j,0)   = u1
               jm2 = j - 2

               do 310 l = ugl, jm2
                  jml = j - l
                  jmlm1 = jml - 1
                  u = c*a(l,jmlm1) - s*a(l,jml)
                  a(l,jml) = s*a(l,jmlm1) + c*a(l,j-l)
                  a(l,jmlm1) = u
 310           continue

               if (j .ne. kpr) then
                  a(jmmm1,m) = c*a(jmmm1,m) - s*g
               end if
               if (n - j .lt. m - 1) then
                  maxl = n - j
               else
                  maxl = m - 1
               end if

               do 320 l = 1, maxl
                  lp1 = l + 1
                  u = c*a(jm1,lp1) - s*a(j,l)
                  a(j,l) = s*a(jm1,lp1) + c*a(j,l)
                  a(jm1,lp1) = u
 320           continue

               if (j + m .le. n) then
                  g = -s * a(j,m)
                  a(j,m) = c * a(j,m)
               end if

               if (matz) then
                  do 330 l = 1, nm
                     u = c*z(l,jm1) - s*z(l,j)
                     z(l,j) = s*z(l,jm1) + c*z(l,j)
                     z(l,jm1) = u
 330              continue
               end if

 300        continue
 200     continue
 100  continue
      a(n,1) = (0.0D0, 0.0D0)
      end subroutine zbandr
