!
! rbnddia --- diagonalizes the band matrix computed by the
!       real symmetric band Lanczos
! Hans-Dieter Meyer
!
!
! Copyright (c) 2003 Hans-Dieter Meyer
! Copyright (c) 2003 Thomas Sommerfeld
! Copyright (c) 2004-2006 Christian Buth
!
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version.
!
! This library is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! Lesser General Public License for more details.
!
! You should have received a copy of the GNU Lesser General Public
! License along with this library; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
!
!
c
c     diagonalizes the bandmatrix computed by the block lanczos
c     and purge the spectrum
c
c     this is not difficult but the purging and output selection
c     is quite complex; in addition there are many fossile structures
c     of Dieter or Uwe and some of the aux arrays are used for several
c     purposes
c
c     n        matrix dimension = no. of lanczos iterations
c     np       half band width  = blocksize in lanczos
c     t        bandmatrix
c     z        contains the first and last np components
c              of the eigenvectors of t
c     a,e,e2   aux arrays
c     d        contains the eigenvalues of t
c     thresh1  threshold for eigenvalue output
c     thresh2  threshold for output of coefficients
c     thresh1 > thresh2 is assumed
c
c     the following parameters are needed to compute eigenvectors.
c     in the 1st version all eigenvectors of t were computed for
c     this purpose and z(n,n); in this version the eigenvectors are
c     computed using inverse iteration and z(np2,n), i.e., zdim is
c     always np2
c
c     neigenv          no. of eigenvectors
c     evlist           is the list of eigenvectors to be computed;
c                      if evlist(0)=0 the eigenvectors are selected
c                      via a max overlap criterion (default)
c     au, al, b, ipvt  are aux arrays for the inverse iteration
c                      they are not refered if neigenv = 0
c
c     iparam is an arry with flags
c     iparam(1) = dieterout (output in dieters format)
c
c     the old output of dieter (first and last n eigenvalues) is
c     controlled via dieterout; my output (all eigenvalues with a
c     certain overlap with the startspace) is controlled via
c     thresh1
c     -------------------------------------------------------------
c
      subroutine rbnddia (n, np, t, a, d, e, e2, z, thresh,
     $     neigenv, evlist, au, al, b, ipvt, iparam)
      use physconst
      implicit none
c
      integer maxn, maxnp
      parameter (maxn = 10000, maxnp = 200)
c
      integer n, np, neigenv, evlist(neigenv), ipvt(n), iparam(*)
      real*8 t(np+1,n), a(n,np+1), d(n), e(n), e2(n), z(2*np,n)
      real*8 au(n,2*np+1), al(n,2*np+1), b(n)
      real*8 thresh(*)
c
      integer np1, np2, zdim, mmm, i, j, iwgt(maxnp), nwgt
      integer ninvi, ind2(maxn), indx, ngt, wrterf
      real*8 wgt(maxnp), thresh1, thresh2, eta
c
c     Dieters Programmteile:
c
      integer dieterout
      integer nsp, nmu, isp, jnd, ind(maxn), mlt(maxn), lsp(maxn)
      integer ii, jj, k, kk, ia, ic, jc, icnv, jcnv
      real*8 aa, sum, sup, ak, a1k, xm
      real*8 ovl, thrs, y1, y2, x1, x2
c
      real*8 ddot
      external ddot
c
      include 'iosetting.inc'
c
c     -----------------------------------------------------------
      np1 = np + 1
      np2 = 2 * np
      mmm = np2 + 1
      zdim = np2
      ninvi = 1
      dieterout = iparam(1)
      wrterf = iparam(2)
      thresh1 = thresh(1)
      thresh2 = thresh(2)
      eta     = thresh(3)
c
      write (6,4000)
 4000 format (//'  Diagonalization of the band-matrix T'/)
      write (6,*) '    matrix dimension : ', n
      write (6,*) '    half band width  : ', np
      if (n .gt. maxn .or. np .gt. maxnp) then
         write(6,*)
         write(6,*) 'maxdim = ', maxn
         write(6,*) 'maxnp  = ', maxnp
         write(6,*) 'change parameters in banddia.'
         stop
      endif
c     ---------------------------------------
c     restore t as appropriate for bandrm
c     note that this is different from zbandr
c     ---------------------------------------
      do i = 1, np1
         do j = 1, n + 1 - i
            ii = j + i - 1
            jj = np1 + 1 - i
            a(ii, jj) = t(i,j)
         enddo
      enddo
c
c     call bandr in any case to initialize z
c
      call bandrm(zdim, n, np1, a, d, e, e2, .true., z)
c
      call tql2m(zdim, n, d, e, z, i)
      if (i.ne.0) then
         write (6,*) '====  Error in tql2  ==='
         write (6,*) i,
     $        '-th Eigenvalue not converged after 30 Iterations'
         stop
      endif
c
c     write the first unpurged eigenvalues to stdout
c
      write (6,'(/,A)') '  The lowest 10 eigenvalues of T:'
      do i = 1, n ! 10
         write (6,'(4x,I2,2x,F15.8)') i, d(i) * HartreetoeV
      enddo
c
c     T has been diagonalized, now purge the spectrum and
c     prepare a nice output
c
C     ----------------------------------------------------------
C     DETERMINE THE INDEX-VECTOR 'IND' IN ORDER TO REMOVE
C     MULTIPLE AND SPURIOUS EIGENVALUES.
C     ----------------------------------------------------------
      NSP = 0
      NMU = 0
      ISP = 0
      JND = 0
      J   = 0
C
  50  J   = J + 1
      IF( J .GT. n )                               GOTO  40
c      CALL rMULT( Z(1,J), Z(1,J), NP , AA )
      aa=ddot(np,Z(1,J),1,Z(1,J),1)
      NSP = NSP + 1
      ISP = ISP + 1
C.....TEST OF SPURIOUS EIGENVALUE.
      IF( AA .LT. 1.D-14 )                     GOTO  50
      NSP = NSP - 1
      ISP = ISP - 1
      JND = JND + 1
      IND(JND) = J
      MLT(JND) = 0
      LSP(JND) = ISP
      ISP      = 0
C
      IF( J .EQ. n )                               GOTO  40
      THRS = 5.D-7*( 1.D0+DABS(d(J)) )
C.....TEST OF MULTIPLE EIGENVALUE.
      IF( DABS( d(J) - d(J+1) ) .GT. THRS )           GOTO  50
      SUM = AA
      SUP = AA
      KK  = 0
      IA  = 0
C
      DO  41  K = 1, n - J
      OVL = 1.D-5
      IF( DABS( d(J+IA) - d(J+K) ) .GT. THRS )        GOTO  42
c      CALL rMULT( Z(1,J+K), Z(1,J+K), NP , AK )
      ak=ddot(np,Z(1,J+K),1,Z(1,J+K),1)
      IF( AK .LE. 1.D-14 )  THEN
         KK     = K
         GOTO  41
      ELSE IF( AK/SUP + SUP/AK  .LT.  2.D+6 )    THEN
           OVL = 1.D-7
           IF( DABS(d(J+IA)-d(J+K)) .GT. 1.D-2*THRS )   GOTO  42
      END IF
c      CALL rMULT( Z(1,J+K), Z(1,J+IA), NP , A1K )
      a1k=ddot(np,Z(1,J+K),1,Z(1,J+IA),1)
      A1K = A1K*A1K/(SUP*AK)
      IF( DABS( A1K-1.D0 ) .LE. OVL )            THEN
         SUM = SUM + AK
         IF( AK .GT. SUP )  THEN
           SUP = AK
           IA  =  K
         END IF
         KK = K
      ELSE
        GOTO  42
      END IF
  41  CONTINUE
C
  42  d(J) = d(J+IA)
      DO  43  I = 1, NP
         Z(   I,J) = Z(   I,J+IA)*DSQRT(SUM/SUP)
         Z(NP+I,J) = Z(NP+I,J+IA)
  43  CONTINUE
C
      NMU = NMU + KK
      J   =  J  + KK
      MLT(JND) =  KK
      GOTO  50
C
  40  CONTINUE
C
C.....REMOVE SPOURIOUS AND MULTIPLE EIGENVALUES.
      DO  48  J = 1, JND
         I = IND(J)
         d(J) = d(I)
         DO  49  K = 1, NP2
            Z(K,J) = Z(K,I)
 49      continue
 48   CONTINUE
C
      write (6,'(/,A)') '  Information on purging:'
  55  PRINT 905, JND, n, int(100.0d0*dble(jnd)/dble(n))
 905  FORMAT(4x,'There are',I5,' relevant residues out of',I5,
     $     '   (',I3,'%)')
      write(6,'(4x,A,I4,6x,A,I4)') 'N_spurios = ', NSP,
     $     'N_multiple = ', NMU
C
C
C.....TRANSFORM THE EIGENVECTORS(FIRST NP ROWS) TO OBTAIN THE RESIDUES.
C.....STORE THE RESULT IN THE FIRST NP ROWS OF THE EIGENVECTOR MATRIX Z.
C.....MULTIPLY :  EIGENVECTORS(LAST ROWS) * 'ERROR-TRIANGLE'
C.....STORE THIS RESULT IN THE LAST ROWS OF THE EIGENVECTOR MATRIX Z.
C
       DO  35  I  = 1, NP
         XM = 0.D0
         DO  36  K = I, NP
           KK = K + NP
           IC = K + n - NP
           JC = I - K + NP1
           XM = XM + Z(KK,J)*T(JC,IC)
  36     CONTINUE
  35   Z(I+NP,J) = XM
  30  CONTINUE
C
C.....PRINT MEAN RESIDUES (X), EIGENVALUES (W), ERROR-EST.(Y) AND MULT.
C
      if (dieterout .eq. 0) GOTO 329
      ICNV = 0
      JCNV = 0
      PRINT 900
 900  FORMAT(//2X,'E I G E N V A L U E S'/2X,125('-')//2
     $     ('   NO.  MEAN-RES.*100  EIGEN-VALUE',3x
     $     ,'ERROR     M   S',9x) // 2X,125('-')/ )
      DO  330  J = 1, (JND+1)/2
         Y1 = 0.D0
         Y2 = 0.D0
         K  = JND + 1 - J
         DO  332  I = 1, NP
            Y1 = Y1 + Z(NP+I,J)**2
            Y2 = Y2 + Z(NP+I,K)**2
 332     continue
         Y1 = dsqrt(Y1)
         Y2 = dsqrt(Y2)
         IF(Y1.LT.1.D-9 .AND. J.NE.JND/2+1) ICNV = ICNV + 1
         IF(Y2.LT.1.D-9)  ICNV = ICNV + 1
         IF(Y1.LT.1.D-6 .AND. J.NE.JND/2+1) JCNV = JCNV + 1
         IF(Y2.LT.1.D-6)  JCNV = JCNV + 1
         IF( J .GT. dieterout )   GOTO  330
         X1 = 0.D0
         X2 = 0.D0
         DO  331  I = 1, NP
            X1 = X1 + Z(I,J)**2
            X2 = X2 + Z(I,K)**2
 331     continue
         X1 = DSQRT(X1)
         X2 = DSQRT(X2)
         PRINT 901, J,X1,d(J),Y1,MLT(J),LSP(J), K,X2,d(K),Y2,MLT(K)
     $        ,LSP(K)
 901     FORMAT(1X, 2( I4, 2PF12.6, 0PF16.9, 1PE11.2, 2I4, 11X ) )
 330  CONTINUE
      PRINT 990
 990  FORMAT(/2('   NO.  MEAN-RES.*100  EIGEN-VALUE',3x,
     $     'ERROR     M   S         '),//,2X,125('-')/ )
      PRINT 902, ICNV, 100.D0*ICNV/DBLE(JND), 100.D0*ICNV/DBLE(n)
      PRINT 903, JCNV, 100.D0*JCNV/DBLE(JND), 100.D0*JCNV/DBLE(n)
 902  FORMAT(/,'    NUMBER OF CONVERGED (1.D-9) EIGENVALUES =',
     #         I4, F11.2, ' %  (JND) ', F9.2,' %  (JEND) ' )
 903  FORMAT(/,'    NUMBER OF CONVERGED (1.D-6) EIGENVALUES =',
     #         I4, F11.2, ' %  (JND) ', F9.2,' %  (JEND) ' )
 329  CONTINUE
c     ----------------------------------------------------------------
c     print eigenvalues mit mean-res > thresh1
c     collect the corresponding indices in ind2()
c     and the mean res in the array e2  (aux for bandrm)
c     --------------------------------------------------
      write(6,400) thresh1
 400  format(//2x,60('-')/'  Eigenvalues with mean-res > ',F6.3,/,/,
     $     '  no.    mean-res.     eigenvalue',7x,'error estimate'/2x,
     $     60('-'))
      ngt = 0
      do j = 1, jnd
         x1 = 0
         do i = 1, np
            x1 = x1 + z(i,j)**2
         enddo
         x1 = dsqrt(x1)
         if (x1 .gt. thresh1) then
            ngt = ngt + 1
            ind2(ngt) = ind(j)
            e2(ngt) = x1
            y1 = 0.0d0
            do i = 1, np
               y1 = y1 + z(np+i,j)**2
            enddo
            y1 = dsqrt(y1)
            write(6,420) j, x1, d(j), y1, ind(j)
 420        format(I5,2x,F8.4,4x,F14.8,4x,1PE11.2,'  (',I5,')')
         endif
      enddo
c     ---------------------------------------------------------
c     write the eigenvalue list to a separate file for eta-runs
c     ---------------------------------------------------------
      if (wrterf .eq. 1) then
         open (ifile,file='ETARUN',access='SEQUENTIAL',form='FORMATTED')
         write(ifile,'(1PE18.8)') eta
         write(ifile,'(I6)') ngt
         do j = 1, ngt
            write(ifile,'(I4,1PE18.8)') j, d(ind2(j))
         enddo
         close (ifile)
      endif
c     ---------------------------------------------------------
c     compute eigenvectors of Z using invers iteration
c     the eigenvectors are written to disk: ZMAT
c     if evlist is undefined the first neigenv in ind2 are used
c     ---------------------------------------------------------
      if (neigenv .gt. 0) then
         if (evlist(1) .eq. 0) then
            if (neigenv .gt. ngt) then
               write(6,*) 'Only ', ngt, ' vectors can be computed.'
               neigenv = ngt
            endif
            do i = 1, neigenv
               evlist(i) = ind2(i)
            enddo
         endif
         write(6,430) (evlist(i), i = 1, neigenv)
 430     format(/,'The following eigenvectors are computed:'/,
     $        20(4x,10(2x,I5),/))
         open (ifile,file='ZMAT',access='SEQUENTIAL',form='UNFORMATTED')
         write(ifile) n, neigenv
         do j = 1, neigenv
            do i = 1, n
               b(i) = 0.0d0
            enddo
            do i = 1, np
               b(i)          = z(i, evlist(j))
               b(n - np + i) = z(np + i, evlist(j))
            enddo
            indx = 0
            do i = 1, jnd
               if (evlist(j) .eq. ind(i)) then
                  indx=i
               endif
            enddo
            if (indx .eq. 0) then
               write(6,*) 'Index ', evlist(j), ' has not been found.'
               stop
            endif
! CB            call invit(n,np,np1,mmm,d(indx),t,au,al,b,ipvt,
! CB     $           ninvi,.false.)
            stop 'no invit!'
            write(ifile) (b(i), i = 1, n)
         enddo
         close(ifile)
      endif
c     -----------------------------------------------------
c     print weights of start vectors for mean-res > thresh2
c     -----------------------------------------------------
      if (thresh2 .lt. 1.0d0) then
         write(6,'(//,A,F6.3,/)') '  weights (C*C) for m-r > ', thresh2
         do i = 1, jnd
            x1 = 0
            do j = 1, np
               x1 = x1 + z(j,i)**2
            enddo
            x1 = dsqrt(x1)
            if (x1 .gt. thresh2) then
               nwgt = 0
               do j = 1, np
                  x1 = z(j,i) **2
                  if (x1 .gt. 0.001d0) then
                     nwgt = nwgt + 1
                     iwgt(nwgt) = j
                     wgt(nwgt) = x1
                  endif
               enddo
               write(6,'(X,I4,A,F14.8)') i, '-th root  E = ', d(i)
               write(6,480) (wgt(j), iwgt(j), j=1, nwgt)
 480           format(6(4x,8(2x,F5.3,'(',I2,')'),/))
               write(6,*)
            endif
         enddo
      endif
c
      return
      end
