c     ----------------------------------------------------------------
c
c     BAND LANCZOS for complex symmetric matrices
c
c     large parts of this routine are taken from the block lanczos
c     program by H-D. Meyer and U. V. Riss
c
c     a matrix A is transformed to band-diagonal form T
c     A is only refered to via the subroutine zmatblk, which
c     has to perform the matrix times vector-block operation
c
c     ndim     dimension of the matrix A
c     np       block-size for the iteration --> band width of T
c     niter    no. of iterations (matrix*vector operations)
c     iparam   array of additional parameters
c     iparam(1)  jstart     1st iteration  (can be > 1 in restarts)
c     iparam(2)  jend       index of last iteration (output)
c     iparam(3)  neigenv    abs of no. of eigenvectors to be computed
c                neigenv =  0  : normaler Lanczos
c                neigenv = -1  : Vorlauf fuer Eigenvektoren
c                neigenv >  0  : Eigenvektorlauf
c     iparam(4)  restart
c     iparam(5)  wrtrsf    write a restart file
c     iparam(6)  wrttmat   write T to a file
c     iparam(7)  ncartsv   no. of cartesian start-vectors
c                if the start-vectors are cartesian this is used
c                to normalize the lanczos-vectors on the start-block
c
c     t        contains on output the computed band matrix
c     p        contains 3*np vectors needed in the iteration
c              on input pr/i(1,i,2) is used as the i-th start-vector
c              (it's not my fault)
c
c     z and ev are only refered to if iparam(3) > 0
c     z        contains some eigenvectors of T from a previous run
c     ev       contains on output the corresponding eigenvectors of A
c
c     icartsv  contains the indices of the cartesian start-vectors
c
c     external zmatblk (ndim, np, pold, pnew, work, jdiesel)
c     is the mat times vector routine
c     work contains space for zmatvec and is not referenced in the lanczos
c
c     THE SUBROUTINE INDEXL COMPUTES THE CURREND INDEX OF THE LANCZOS-
c                           VECTOR CONTAINED IN THE ARRAY P.
c
c     -----------------------------------------------------------------
c
      subroutine zblanc (ndim, np, niter, neigenv, ncartsv, iparam, t,
     $     p, z, ev, icartsv, zmatblk, mtxwork, jdiesel)
c
      implicit none
c
      integer ndim, np, niter, neigenv, ncartsv, jdiesel
      integer iparam(10), icartsv(ncartsv)
      complex*16 p(ndim,np,3), t(np+1,niter)
      complex*16 z(niter,neigenv), ev(ndim,neigenv), mtxwork(*)
      external zmatblk
c
      integer jstart, jend
      integer restart, wrtrsf, wrttmat
      integer np1, np3
c
      complex*16 y
      integer i, j, k, nrun, ndone
      integer jj, ii, jn, in, jo, io, jjc, iic, im, ip, jp
      integer jiter, jko
      logical da
c
      complex*16 zdotu
      external zdotu
c
      include 'iosetting.inc'
c
c     -------------------------------------------------------------
c     preliminaries
c     -------------------------------------------------------------
      write (6,100)
 100  format (//,2X,50('-')/,2X,'Band-Lanczos (complex)'/)
      jstart  = iparam(1)
      jend    = iparam(2)
      restart = iparam(4)
      wrtrsf  = iparam(5)
      wrttmat = iparam(6)
      np1 = np + 1
      np3 = 3 * np
      jstart = 1
c     ---------------------------------------------------------
c     read the restartfile
c     --------------------
      if (restart .ne. 0) then
         write (6,'(4x,A)') 'Restart from file'
         inquire(file='RESTART', exist=da)
         if (.not. da) then
            write(6,*) 'RESTART File is missing.'
            stop
         endif
         open (ifile,file='RESTART',access='SEQUENTIAL',
     $        form='UNFORMATTED')
         read(ifile) jend, j
         if (j .ne. ndim) then
            write(6,*) 'Inconsistancy:'
            write(6,*) 'Ndim(Inputfiles)  = ', ndim
            write(6,*) 'Ndim(Restartfile) = ', j
            stop
         endif
         write(6,'(6x,A,I5)') 'Iterations done so far  ', jend
         write(6,'(6x,A,I5)') 'Iterations to be done   ', niter-jend
         if (jend .ge. niter) then
            stop 'Nothing to do'
         endif
         read(ifile) ((t(i,j), i=1, np1),  j=1, jend)
         read(ifile) ((p(i,j,1), i=1, ndim), j=1, np),
     $            ((p(i,j,2), i=1, ndim), j=1, np)
         read(ifile) ncartsv
         read(ifile) (icartsv(i), i=1, ncartsv)
         close (ifile)
         jstart = jend + 1
      endif
c     ----------------------------
c     print Iterations and runtype
c     ----------------------------
      write (6,'(4x,A,I5,A,I5)') 'Iterations from ',jstart,' to ',niter
      if (neigenv .gt. 0) then
         write(6,'(4X,A,I4,A)') 'This is a second run computing ',
     $        neigenv,' eigenvectors'
      endif
      write(6,*)
c     ---------------------------------------------------------------
c     prepare for eigenvector run
c     ---------------------------
      if (neigenv .gt. 0) then
c     --------------------------------------
c     read the ZMAT file to z(niter,neigenv)
c     this file is created by banddia after
c     a first lanczos requiring eigenvectors
c     --------------------------------------
        inquire(file='ZMAT', exist=da)
         if (.not. da) then
            write(6,*) '**** The File ZMAT does not exist.'
            stop
         endif
         open (ifile,file='ZMAT',access='SEQUENTIAL',
     $        form='UNFORMATTED')
         read(ifile) i, j
         if (i .ne. niter .or. j .ne. neigenv) then
            write(6,*) '***************   WARNING  *****************'
            write(6,*) ' ZMAT file _may_ be corrupted.'
            write(6,*) ' niter, neigenv (Args) = ', niter, neigenv
            write(6,*) ' zdim,  neigenv (File) = ', i, j
            write(6,*) ' (niter must be too small in a restart set up)'
            write(6,*)
            if (j .lt. neigenv .or. i .lt. niter) then
               stop
            endif
         endif
         do j = 1, neigenv
            read(ifile) (z(i,j), i = 1, niter)
         enddo
         close (ifile)
c     -------------------------------
c     initialize or read eigenvectors
c     -------------------------------
         if (restart .eq. 0) then
            do i = 1, neigenv
               do j = 1, ndim
                  ev(j,i) = (0.0d0, 0.0d0)
               enddo
            enddo
            do j = 1, np
               jiter = jstart - 1 + j
               do i = 1, neigenv
                  do jko = 1, ndim
                     ev(jko,i) = ev(jko,i)+p(jko,j,2)*z(jiter,i)
                  enddo
               enddo
            enddo
c
         else
c     ------------------------------------------------------
c     restart = 1
c     read eigenvectors from the EIGENV file of the last run
c     ------------------------------------------------------
            inquire(file='EIGENV', exist=da)
            if (.not. da) then
               write(6,*) 'EIGENV File is missing.'
               stop
            endif
            open (ifile,file='EIGENV',access='SEQUENTIAL',form
     $           ='UNFORMATTED')
            read(ifile) i, j
            if (i .ne. neigenv .or. j .ne. ndim) then
               write(6,*) 'EIGENV file may be corrupted.'
               write(6,*) 'ndim, neigenv (Args) = ', ndim, neigenv
               write(6,*) 'ndim, neigenv (File) = ', j, i
               stop
            endif
            read(ifile) i
            if (i .ne. 2) then
               write(6,*) 'itype = ',i,' ?????'
               stop
            endif
            do j = 1, neigenv
               read(ifile) (ev(i,j), i = 1, ndim)
            enddo
            close(ifile)
c     ------------------------------------------
c     contributions of the restart vectors which
c     have NOT been added in the previous run!
c     ------------------------------------------
            do j = 1, np
               jiter = jstart - 1 + j
               do i = 1, neigenv
                  do jko = 1, ndim
                     ev(jko,i) = ev(jko,i) + p(jko,j,2) * z(jiter,i)
                  enddo
               enddo
            enddo
         endif
      endif
c     ============================================================
c     end of eigenvector preparations, now start the
c
c     Main Loop of the Lanczos Iteration
c     ----------------------------------
      nrun = niter - jstart + 1
      ndone = 10
c
      do 1  JJ = JSTART, NITER
c
         CALL INDEXL(JJ-JSTART+NP, NP, jn, IN )
c
c     when neccessary do the next matrix times block operation
c
         if (MOD(JJ-JSTART,NP)  .EQ. 0) then
            io = mod(in+1, 3) + 1
            call zmatblk(ndim,np,p(1,1,io),p(1,1,in),mtxwork,jdiesel)
         endif
c
c     Band-Lanczos algorithm as implemented by H-D. Meyer
c
         do II = MAX0(JJ-NP,1), JJ-1
            CALL INDEXL(II-JSTART+NP3, NP, JO, IO )
            JJC = JJ - II + 1
c            do K = 1, NDIM
c               P(k,JN,IN) = P(k,JN,IN) - P(k,JO,IO)*T(JJC,II)
c            enddo
            call zaxpy(ndim,-t(jjc,ii),p(1,jo,io),1,p(1,jn,in),1)
         enddo
c
         do II = JJ, JJ + NP - 1
            CALL INDEXL(II-JSTART, NP, JO, IO )
c            Y = (0.0d0, 0.0d0)
c            do K = 1, NDIM
c               Y = Y + P(k,JO,IO) * P(k,JN,IN)
c            enddo
            y = zdotu(ndim,p(1,jo,io),1,p(1,jn,in),1)
c            do K = 1, NDIM
c               P(k,JN,IN) = P(k,JN,IN) - P(k,JO,IO)*Y
c            enddo
            call zaxpy(ndim,-y,p(1,jo,io),1,p(1,jn,in),1)
            IIC = II - JJ + 1
            T(IIC,JJ) = Y
         enddo
c     -------------------------------------------------
c     orthogonalization wrt cartesian start-vectors
c     Vorsicht bei restart; icartsv muss definiert sein
c     -------------------------------------------------
         if (ncartsv .gt. 0) then
            do k = 1, np
               p(icartsv(k),jn,in) = (0.0d0, 0.0d0)
            enddo
         endif
c     --------------------
c     normalize new vector
c     --------------------
c         Y = (0.0d0, 0.0d0)
c         do K = 1, NDIM
c            Y = Y + P(k,JN,IN)**2
c         enddo
         y = zdotu(ndim,p(1,jn,in),1,p(1,jn,in),1)
         y = zsqrt(Y)
         T(NP1,JJ) = Y
         y = 1.0d0 / y
c         do K = 1, NDIM
c            P(k,JN,IN) = P(k,JN,IN) * y
c         enddo
         call zscal(ndim,y,p(1,jn,in),1)
c     -------------------------------------------------------
c     sum contributions of this iteration to the eigenvectors
c     -------------------------------------------------------
         if (neigenv .gt. 0) then
            jiter = np + jj
c
c     in the eigenvector calculation the very last np iterations
c     do not contribute to the eigenvectors;
c     but in case of a later restart these iterations have to be done
c
            if (jiter .le. niter) then
               do i = 1, neigenv
                  do jko = 1, ndim
                     ev(jko,i)=ev(jko,i)+z(jiter,i)*p(jko,JN,IN)
                  enddo
               enddo
            endif
         endif
c
c     check how much has been done
c
         if ((jj-jstart+1)*100 / nrun .ge. ndone) then
            write(6,'(6X,I3,A)') ndone, '%'
            call flush(6)
            ndone = ndone + 10
         endif
c
 1    CONTINUE
c     ------------------------------------------------------------
c     End of the Block-Lanczos iteration
c     jend was used when the main-loop was left because of
c     a timelimit (IMB 30/90 version); this has been thrown out
c     ------------------------------------------------------------
      write(6,999)
 999  FORMAT(/,2X,'Lanczos finished',/,2X,50('-'))
      jend = niter
c     ---------------------------------------------------
c     write the bandmatrix T(np1, niter) to the file TMAT
c     for convergence and other checks
c     ---------------------------------------------------
      if (wrttmat .eq. 1) then
         open(unit=ifile,file='TMAT',access='SEQUENTIAL',
     $        form='UNFORMATTED')
         write (ifile) jend, np, niter
         write (ifile) t
         close (ifile)
      endif
c     -----------------------------------------------
c     write the restart file
c     this part was written by Dieter;
c     i do not really understand it, however it works
c     -----------------------------------------------
      if (wrtrsf .eq. 1) then
         open (ifile,file='RESTART',access='SEQUENTIAL',
     $        form='UNFORMATTED')
         CALL INDEXL(JEND-JSTART+NP, NP, JP, IP )
         IO  = MOD(IP+1,3) + 1
         IM  = MOD(IP  ,3) + 1
         write(ifile) jend, ndim
         write(ifile) ((T(I,J), I=1,NP1) ,J=1,JEND)
         write(ifile) ((P(i,j,IM), i=1,NDIM), j=JP+1,NP),
     $             ((P(i,j,IO), i=1,NDIM), j=1,   NP),
     $             ((P(i,j,IP), i=1,NDIM), j=1,   JP)
         write(ifile) ncartsv
         write(ifile) (icartsv(i), i=1, ncartsv)
         close (ifile)
      endif
c
      return
      end
c
c
c     one of Dieter's more serious crimes:
c
c
      SUBROUTINE INDEXL( JJ, NP, J, I )
C
C     AH1:LANCZ.INDEXL
C     THIS ROUTINE COMPUTES THE INDICES J,I OF THE JJ-TH LANCZOS-VECTOR.
C
         KK = JJ/NP
         I  = MOD( KK+1, 3 ) + 1
         J  = JJ + 1 - KK*NP
C
      RETURN
      END
