!
! wannier --- Provide Hartree-Fock band structures from the Wannier program
! Christian Buth
! 27.02.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version.
!
! This library is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! Lesser General Public License for more details.
!
! You should have received a copy of the GNU Lesser General Public
! License along with this library; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
!
!
! Version 1.00: 27.02.2004 (Extracted Wannier specific information from adcbands module)
! Version 1.10: 07.03.2004 (Added the selection of a subset of bands in the Wannier program)
! Version 1.20: 10.07.2004 (Object oriented programming style)
!


!
! Module
!

      module wannier
      use fortrantl
      use lattice
      use typeswannier
      use Vintegrals
      implicit none


!
! Interface
!

      interface
      subroutine initwannierprogram(wb, fileFockTrans, fileVBucket, fileVijkl, fileVIndex, &
     &                              fileFockAll, numorb, numocc)
        use fortrantl
        use lattice
        use typeswannier
        character(len=*), intent(in) :: fileFockTrans, fileVBucket, fileVijkl, fileVIndex, fileFockAll
        integer, intent(out) :: numorb, numocc
        type(twannier), intent(inout) :: wb
      end subroutine initwannierprogram


      subroutine fockall(wb, fileFockAll)
        use fortrantl
        use lattice
        use typeswannier
        character(len=*), intent(in) :: fileFockAll
        type(twannier), intent(inout) :: wb
      end subroutine fockall


      subroutine molfock(wb,jobz,eigenvectors,eigenvalues)
        use fortrantl
        use lattice
        use typeswannier
        character, intent(in) :: jobz
        real(dblprec), intent(out) :: eigenvalues(:),   &
     &                                eigenvectors(:,:)
        type(twannier), intent(in) :: wb
      end subroutine molfock


      subroutine wannierfock(wb, kpoint, eigenvalues)
        use fortrantl
        use lattice
        use typeswannier
        !!!!complex(dblprec), intent(out) :: FockSymmetryAdapted(:,:)
        real(dblprec), intent(out) :: eigenvalues(:)
        type(twannier), intent(in) :: wb
        type(tlatticevector), intent(in) :: kpoint
      end subroutine wannierfock
      end interface


!
! Module procedures
!

      contains


!
! initwannier --- open Wannier files
!

      subroutine initwannier(wb, periodicity, noucs, fileFockTrans, fileVBucket, fileVijkl, fileVIndex, &
     &                       fileFockAll)
      implicit none

      character(len=*), intent(in) :: fileFockTrans, fileVBucket, &
     &          fileVijkl, fileVIndex, fileFockAll
      integer, intent(in) :: periodicity, noucs
      type(twannier), intent(inout) :: wb

      wb%periodicity  = periodicity
      wb%numUnitCells = noucs

  !!    type(tbrillouinpath) wbdummy
  !!    if (periodicityDim /= 0) then
  !!      call wannierbands(wbdummy, wb%periodicity, fileBands, wb%endVirtual, &
  !!   &                    wb%startOccupied, wb%numOrbitals, .false.)
  !!    end if

      call initwannierprogram(wb, fileFockTrans, fileVBucket, fileVijkl, fileVIndex, &
     &                        fileFockAll, wb%numOrbitals, wb%numOccupied)
      wb%startOccupied = 1                      ! Wannier always outputs all orbitals
      wb%endVirtual    = wb%numOrbitals         ! Wannier always outputs all orbitals
      wb%numVirtual    = wb%numOrbitals - wb%numOccupied

      write (*,*) 'Wannier parameters for the calculation'
      write (*,*) 'startWannierOccupied, endWannierVirtual, numWannierOrbitals, numWannierOccupied, numWannierVirtual'
      write (*,*) wb%startOccupied, wb%endVirtual, wb%numOrbitals, wb%numOccupied, wb%numVirtual
      end subroutine initwannier


!
! exitwannier --- close Wannier files
!

      subroutine exitwannier(wb)
      implicit none

      type(twannier), intent(in) :: wb

      if (wb%periodicity /=0) then
        deallocate(wb%cdirect, wb%idirect)
      end if
      end subroutine exitwannier


!
! wannierepsilon --- Wannier orbital energies
!

      real(dblprec) function wannierepsilon(orbital)
      implicit none

      integer, intent(in) :: orbital

      ! Obtain Wannier orbital energies, central cell R = (0,0,0)
      wannierepsilon = fallfind(orbital, 1, orbital)
      end function wannierepsilon


!
! wannierOrbital --- obtain the Wannier orbital index, as used in the Wannier program,
!       in terms of lattice vector R and band index n
!

      integer function wannierOrbital(wb, R, n)
      implicit none

      integer, intent(in) :: R, n
      type(twannier), intent(in) :: wb

      wannierOrbital = R + (n-1) * wb%numUnitCells
      end function wannierOrbital
      end module wannier
