!
! orbcoeff --- analyse the contributions to the orbital coefficients
!       of the atomic basis functions
! Christian Buth
! 04.09.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!


!
! Program
!

      program orbcoeff
      use fortrantl
      implicit none

      integer norbs, &                                  ! Number of orbitals
     &        nocc,  &                                  ! Number of occupied orbitals
     &        nvirt, &                                  ! Number of virtual orbitals
     &        npart, &                                  ! Number of orbitals of partial occupation
     &        nbas,  &                                  ! Number of basis functions
     &        natoms, &                                 ! Number of atoms per unit cell
     &        ncells, &                                 ! Number of unit cells
     &        nbascell,     &                           ! Number of basis functions per unit cell
     &        nbasatom(50), &                           ! Number of basis functions for each atom per unit cell
     &        shiftcell,    &
     &        shiftbas,     &
     &        ibeg,         &
     &        i, j, k, l, dummy

      real(dblprec) norm, contrib                       ! Norm of orbital coefficients vector
      real(dblprec), allocatable :: occupancy(:), &     ! Occupation numbers of the orbitals
     &                              coeff(:,:)          ! Coefficients of the orbitals


      write (*,*) 'Orbital coefficients analysis'
      write (*,*) '(c) 2004 Christian Buth'
      write (*,*)


        natoms = 4
        ncells = 9
        nbasatom(1) = 14
        nbasatom(2) = 5
        nbasatom(3) = 14
        nbasatom(4) = 5

        nbasatom(1) = 5
        nbasatom(2) = 1
        nbasatom(3) = 5
        nbasatom(4) = 1

      ! Read orbital coefficients file
      read (*,*) norbs, nbas, dummy

      allocate(occupancy(norbs), coeff(nbas,norbs))
      read (*,*) (occupancy(i), i=1, norbs)
      read (*,*) ((coeff(i,j), i=1, nbas), j=1, norbs)

      ! Analyse header
      nocc  = 0
      nvirt = 0
      npart = 0
      do i=1, norbs
        if (occupancy(i) == 2.0) then
            nocc  = nocc  + 1
        else if (occupancy(i) == 0.0) then
            nvirt = nvirt + 1
        else
            npart = npart + 1
        end if
      end do
      write (*,*) 'Number of           orbitals ', norbs
      write (*,*) '                    occupied ', nocc
      write (*,*) '                     virtual ', nvirt
      write (*,*) '          partially occupied ', npart
      write (*,*) '             basis functions ', nbas
      nbascell = sum(nbasatom(1 : natoms))
      write (*,*) '                       atoms ', natoms
      write (*,*) '                  unit cells ', ncells
      write (*,*) '          basis functions per unit cell ', nbascell
      write (*,*)

      ! Analyse obitals
      do i=1, norbs
        write (*,*) 'Orbital No. ', i, ' occupancy ', occupancy(i), ':'
        norm = dot_product(coeff(:,i), coeff(:,i))
        write (*,*) 'Norm of orbital coefficients vector ', norm

        shiftcell = 0
        !do j=1, ncells
      !  do j=1, 1
          write (*,*) 'Analysing orbital coefficients in unit cell ', j

          shiftbas = 0
          do k=1, natoms

            contrib = 0
            do l = shiftbas, shiftbas + nbasatom(k) - 1
              do j=1, ncells
                ibeg = l * ncells + j
                contrib = contrib + coeff(ibeg, i)**2
              end do
            end do

            write (*,*) 'Contribution of atom No. ', k, ' is ', contrib / norm
            shiftbas = shiftbas + nbasatom(k)
          end do

          write (*,*)
       ! end do

        write (*,*)
      end do

      deallocate(occupancy, coeff)
      end program orbcoeff
