!
! pointcharges_HF --- generate point charges in MOLPRO format for an infinite HF chain
! Christian Buth
! 21.10.2004
!
!
! Copyright (c) 2004-2006 Christian Buth
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 2 of the License, or
! (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
!
!
! Version 1.00: 22.10.2004 (Produce geometry and point charges for MOLPRO)
!


      program pointcharges_HF
      use fortrantl
      use physconst
      implicit none


      type tlatticevector
        real(dblprec) v(3)
      end type tlatticevector


      character(2) element(100)
      character(20) charge(100)
      integer numLattices, i, currLat
      real(dblprec) latticeconst(100) !, innerradius, outerradius
      type(tlatticevector) atompos(100), point, displace

      ! Parameter for HF
      numLattices = 4
      latticeconst(1) = 4.33013
      latticeconst(2) = 4.33013
      latticeconst(3) = 4.33013
      latticeconst(4) = 4.33013
      atompos(1)%v = (/0.0, -0.625, 0.0/)
      atompos(2)%v = (/0.796743, -0.165, 0.0/)
      atompos(3)%v = (/2.16506, 0.625, 0.0/)
      atompos(4)%v = (/2.96181, 0.165, 0.0/)
      charge(1) = 'qF'
      charge(2) = 'qH'
      charge(3) = 'qF'
      charge(4) = 'qH'
      element(1) = 'F'
      element(2) = 'H'
      element(3) = 'F'
      element(4) = 'H'

      ! Write geometry header
      write (*,'(A)') '!'
      write (*,'(A)') '! MOLPRO1992 geometry'
      write (*,'(A)') '!'
      write (*,*)
      write (*,'(A)') 'int'
      write (*,'(A)') 'nosym'

      do i=-5, 4
        do currLat=1, numLattices
          displace%v = (/latticeconst(currLat), 0.0D0, 0.0D0/)
          displace%v = displace%v  * real(i, dblprec)
          point%v = atompos(currLat)%v + displace%v
          write (*,'(A,I0,3A,3(F13.8,A),I0)') ' a', currLat, ',', &
     &          trim(element(currLat)), ',', point%v(1), '*a,',  &
     &          point%v(2), '*a,', point%v(3), '*a     ! Cell R = ', i
        end do
      end do

      ! Write point charge header
      write (*,*)
      write (*,*)
      write (*,'(A)') '!'
      write (*,'(A)') '! MOLPRO1992 point charges'
      write (*,'(A)') '!'
      write (*,*)

      do i=-20, -5
        do currLat=1, numLattices
          displace%v = (/latticeconst(currLat), 0.0D0, 0.0D0/)
          displace%v = displace%v  * real(i, dblprec)
          point%v = atompos(currLat)%v + displace%v
          !point%v = point%v * AngstroemToBohr
          write (*,'(A,3(F13.8,A),A,A,I0)') ' lat, ', point%v(1), '*a,', point%v(2), &
     &          '*a,', point%v(3), '*a, ', trim(charge(currLat)), '    ! Cell R = ', i
        end do
      end do

      end program pointcharges_HF
